@@ -41,26 +41,6 @@
 #include <unistd.h>
 #endif
 
-/* unfortunately, glibc doesn't have a wrapper for the ioprio_set ()
- * syscall, so we have to do it the hard way.  also, it seems some
- * systems don't have <linux/ioprio.h>, so i'll copy the defines here.
- */
-#ifdef HAVE_ASM_UNISTD_H
-#  include <asm/unistd.h>
-#  include <sys/syscall.h>
-#  ifdef __NR_ioprio_set
-#    ifdef HAVE_WORKING_LINUX_IOPRIO_H
-#      include <linux/ioprio.h>
-#    else  /* if !HAVE_WORKING_LINUX_IOPRIO_H */
-#      define IOPRIO_CLASS_SHIFT              (13)
-#      define IOPRIO_PRIO_MASK                ((1UL << IOPRIO_CLASS_SHIFT) - 1)
-#      define IOPRIO_PRIO_VALUE(class, data)  (((class) << IOPRIO_CLASS_SHIFT) | data)
-#      define IOPRIO_WHO_PROCESS              (1)
-#      define IOPRIO_CLASS_IDLE               (3)
-#    endif  /* !HAVE_WORKING_LINUX_IOPRIO_H */
-#  endif  /* __NR_ioprio_set */
-#endif  /* HAVE_ASM_UNISTD_H */
-
 #include <gio/gio.h>
 
 #include <X11/ICE/ICElib.h>
@@ -73,7 +53,6 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <libxfsm/xfsm-splash-engine.h>
 #include <libxfsm/xfsm-util.h>
 
 #include <xfce4-session/xfsm-manager-dbus.h>
@@ -88,9 +67,6 @@
 #include <xfce4-session/xfsm-logout-dialog.h>
 
 
-#define DEFAULT_SESSION_NAME "Default"
-
-
 struct _XfsmManager
 {
   XfsmDbusManagerSkeleton parent;
@@ -117,7 +93,7 @@ struct _XfsmManager
   GQueue          *running_clients;
 
   gboolean         failsafe_mode;
-  GQueue          *failsafe_clients;
+  gint             failsafe_clients_pending;
 
   guint            die_timeout_id;
   guint            name_owner_id;
@@ -155,6 +131,9 @@ enum
 
 static guint manager_signals[LAST_SIGNAL] = { 0, };
 
+int               xfsm_splash_screen_choose (GList            *sessions,
+                                             const gchar      *default_session,
+                                             gchar           **name_return);
 static void       xfsm_manager_finalize (GObject *obj);
 
 static gboolean   xfsm_manager_startup (XfsmManager *manager);
@@ -165,7 +144,8 @@ static void       xfsm_manager_cancel_client_save_timeout (XfsmManager *manager,
 static gboolean   xfsm_manager_save_timeout (gpointer user_data);
 static void       xfsm_manager_load_settings (XfsmManager   *manager,
                                               XfconfChannel *channel);
-static gboolean   xfsm_manager_load_session (XfsmManager *manager);
+static gboolean   xfsm_manager_load_session (XfsmManager *manager,
+                                             XfconfChannel *channel);
 static void       xfsm_manager_dbus_class_init (XfsmManagerClass *klass);
 static void       xfsm_manager_dbus_init (XfsmManager *manager,
                                           GDBusConnection *connection);
@@ -210,7 +190,7 @@ xfsm_manager_init (XfsmManager *manager)
   manager->starting_properties = g_queue_new ();
   manager->restart_properties = g_queue_new ();
   manager->running_clients = g_queue_new ();
-  manager->failsafe_clients = g_queue_new ();
+  manager->failsafe_clients_pending = 0;
 }
 
 static void
@@ -225,21 +205,18 @@ xfsm_manager_finalize (GObject *obj)
 
   g_object_unref (manager->shutdown_helper);
 
-  g_queue_foreach (manager->pending_properties, (GFunc) xfsm_properties_free, NULL);
+  g_queue_foreach (manager->pending_properties, (GFunc) G_CALLBACK (xfsm_properties_free), NULL);
   g_queue_free (manager->pending_properties);
 
-  g_queue_foreach (manager->starting_properties, (GFunc) xfsm_properties_free, NULL);
+  g_queue_foreach (manager->starting_properties, (GFunc) G_CALLBACK (xfsm_properties_free), NULL);
   g_queue_free (manager->starting_properties);
 
-  g_queue_foreach (manager->restart_properties, (GFunc) xfsm_properties_free, NULL);
+  g_queue_foreach (manager->restart_properties, (GFunc) G_CALLBACK (xfsm_properties_free), NULL);
   g_queue_free (manager->restart_properties);
 
-  g_queue_foreach (manager->running_clients, (GFunc) g_object_unref, NULL);
+  g_queue_foreach (manager->running_clients, (GFunc) G_CALLBACK (g_object_unref), NULL);
   g_queue_free (manager->running_clients);
 
-  g_queue_foreach (manager->failsafe_clients, (GFunc) xfsm_failsafe_client_free, NULL);
-  g_queue_free (manager->failsafe_clients);
-
   g_free (manager->session_name);
   g_free (manager->session_file);
   g_free (manager->checkpoint_session_name);
@@ -294,7 +271,7 @@ static gboolean
 xfsm_manager_startup (XfsmManager *manager)
 {
   xfsm_startup_foreign (manager);
-  g_queue_sort (manager->pending_properties, (GCompareDataFunc) xfsm_properties_compare, NULL);
+  g_queue_sort (manager->pending_properties, (GCompareDataFunc) G_CALLBACK (xfsm_properties_compare), NULL);
   xfsm_startup_begin (manager);
   return FALSE;
 }
@@ -450,50 +427,17 @@ xfsm_manager_choose_session (XfsmManager *manager,
                              XfceRc      *rc)
 {
   XfsmSessionInfo *session;
-  GdkPixbuf       *preview_default = NULL;
   gboolean         load = FALSE;
   GList           *sessions = NULL;
   GList           *lp;
-  gchar          **groups;
   gchar           *name;
   gint             result;
-  gint             n;
 
-  groups = xfce_rc_get_groups (rc);
-  for (n = 0; groups[n] != NULL; ++n)
-    {
-      if (strncmp (groups[n], "Session: ", 9) == 0)
-        {
-          xfce_rc_set_group (rc, groups[n]);
-          session = g_new0 (XfsmSessionInfo, 1);
-          session->name = groups[n] + 9;
-          session->atime = xfce_rc_read_int_entry (rc, "LastAccess", 0);
-          session->preview = xfsm_load_session_preview (session->name);
-
-          if (session->preview == NULL)
-            {
-              if (G_UNLIKELY (preview_default == NULL))
-                {
-                  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-                  /* TODO: Turn this into a normal pixbuf? */
-                  preview_default = gdk_pixbuf_new_from_inline (-1, xfsm_chooser_icon_data,
-                                                                FALSE, NULL);
-                  G_GNUC_END_IGNORE_DEPRECATIONS
-                }
-
-              session->preview = GDK_PIXBUF (g_object_ref (preview_default));
-            }
-
-          sessions = g_list_append (sessions, session);
-        }
-    }
-
-  if (preview_default != NULL)
-    g_object_unref (preview_default);
+  sessions = settings_list_sessions (rc);
 
   if (sessions != NULL)
     {
-      result = xfsm_splash_screen_choose (splash_screen, sessions,
+      result = xfsm_splash_screen_choose (sessions,
                                           manager->session_name, &name);
 
       if (result == XFSM_CHOOSE_LOGOUT)
@@ -520,14 +464,13 @@ xfsm_manager_choose_session (XfsmManager *manager,
       g_list_free (sessions);
     }
 
-  g_strfreev (groups);
-
   return load;
 }
 
 
 static gboolean
-xfsm_manager_load_session (XfsmManager *manager)
+xfsm_manager_load_session (XfsmManager   *manager,
+                           XfconfChannel *channel)
 {
   XfsmProperties *properties;
   gchar           buffer[1024];
@@ -556,6 +499,7 @@ xfsm_manager_load_session (XfsmManager *manager)
 
   g_snprintf (buffer, 1024, "Session: %s", manager->session_name);
   xfsm_verbose ("loading %s\n", buffer);
+  xfconf_channel_set_string (channel, "/general/SessionName", manager->session_name);
 
   xfce_rc_set_group (rc, buffer);
   count = xfce_rc_read_int_entry (rc, "Count", 0);
@@ -601,16 +545,19 @@ xfsm_manager_load_failsafe (XfsmManager   *manager,
                             XfconfChannel *channel,
                             gchar        **error)
 {
-  FailsafeClient *fclient;
+  XfsmProperties *properties;
   gchar          *failsafe_name;
-  GdkDisplay     *display;
   gchar           propbuf[4096];
+  gchar          *hostname;
+  gchar          *client_id = NULL;
   gchar         **command;
   gchar           command_entry[256];
-  gchar           screen_entry[256];
+  gchar           priority_entry[256];
+  gint            priority;
   gint            count;
   gint            i;
-  gint            n_screen;
+
+  hostname = xfce_gethostname ();
 
   failsafe_name = xfconf_channel_get_string (channel, "/general/FailsafeSessionName", NULL);
   if (G_UNLIKELY (!failsafe_name))
@@ -634,44 +581,30 @@ xfsm_manager_load_failsafe (XfsmManager   *manager,
       return FALSE;
     }
 
-  display = gdk_display_get_default ();
-
   g_snprintf (propbuf, sizeof (propbuf), "/sessions/%s/Count", failsafe_name);
   count = xfconf_channel_get_int (channel, propbuf, 0);
 
   for (i = 0; i < count; ++i)
     {
+      properties = xfsm_properties_new (client_id, hostname);
       g_snprintf (command_entry, sizeof (command_entry),
                   "/sessions/%s/Client%d_Command", failsafe_name, i);
       command = xfconf_channel_get_string_list (channel, command_entry);
       if (G_UNLIKELY (command == NULL))
         continue;
 
-      g_snprintf (screen_entry, sizeof (screen_entry),
-                  "/sessions/%s/Client%d_PerScreen", failsafe_name, i);
-      if (xfconf_channel_get_bool (channel, screen_entry, FALSE))
-        {
-          for (n_screen = 0; n_screen < XScreenCount (gdk_x11_display_get_xdisplay (display)); ++n_screen)
-            {
-              fclient = g_new0 (FailsafeClient, 1);
-              if (n_screen == 0)
-                fclient->command = command;
-              else
-                fclient->command = g_strdupv (command);
-              fclient->screen = gdk_display_get_screen (display, n_screen);
-              g_queue_push_tail (manager->failsafe_clients, fclient);
-            }
-        }
-      else
-        {
-          fclient = g_new0 (FailsafeClient, 1);
-          fclient->command = command;
-          fclient->screen = gdk_screen_get_default ();
-          g_queue_push_tail (manager->failsafe_clients, fclient);
-        }
-    }
+      g_snprintf (priority_entry, sizeof (priority_entry),
+                  "/sessions/%s/Client%d_Priority", failsafe_name, i);
+      priority = xfconf_channel_get_int (channel, priority_entry, 50);
 
-  if (g_queue_peek_head (manager->failsafe_clients) == NULL)
+      xfsm_properties_set_string (properties, SmProgram, command[0]);
+      xfsm_properties_set_strv (properties, SmRestartCommand, g_strdupv(command));
+      xfsm_properties_set_uchar (properties, GsmPriority, priority);
+      g_queue_push_tail (manager->pending_properties, properties);
+    }
+  g_queue_sort (manager->pending_properties, (GCompareDataFunc) G_CALLBACK (xfsm_properties_compare), NULL);
+
+  if (g_queue_peek_head (manager->pending_properties) == NULL)
     {
       if (error)
         *error = g_strdup (_("The list of applications in the failsafe session is empty."));
@@ -682,6 +615,129 @@ xfsm_manager_load_failsafe (XfsmManager   *manager,
 }
 
 
+int
+xfsm_splash_screen_choose (GList            *sessions,
+                           const gchar      *default_session,
+                           gchar           **name_return)
+{
+  GdkDisplay *display;
+  GdkScreen  *screen;
+  int         monitor;
+  GtkWidget  *chooser;
+  GtkWidget  *label;
+  GtkWidget  *dialog;
+  GtkWidget  *entry;
+  gchar       title[256];
+  int         result;
+
+  g_assert (default_session != NULL);
+
+
+again:
+      display = gdk_display_get_default ();
+      screen = xfce_gdk_screen_get_active (&monitor);
+
+      if (G_UNLIKELY (screen == NULL))
+        {
+          screen  = gdk_display_get_default_screen (display);
+          monitor = 0;
+        }
+
+      chooser = g_object_new (XFSM_TYPE_CHOOSER,
+                              "type", GTK_WINDOW_POPUP,
+                              NULL);
+      gtk_container_set_border_width (GTK_CONTAINER (chooser), 6);
+      xfsm_chooser_set_sessions (XFSM_CHOOSER (chooser),
+                                 sessions, default_session);
+      gtk_window_set_screen (GTK_WINDOW (chooser), screen);
+      gtk_window_set_position (GTK_WINDOW (chooser), GTK_WIN_POS_CENTER);
+      /* Use Adwaita's keycap style to get a meaningful look out of the box */
+      gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (chooser)), "keycap");
+      gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (chooser)), "xfsm-session-chooser");
+      result = gtk_dialog_run (GTK_DIALOG (chooser));
+
+      if (result == XFSM_RESPONSE_LOAD)
+        {
+          if (name_return != NULL)
+            *name_return = xfsm_chooser_get_session (XFSM_CHOOSER (chooser));
+          result = XFSM_CHOOSE_LOAD;
+        }
+      else if (result == XFSM_RESPONSE_NEW)
+        {
+          result = XFSM_CHOOSE_NEW;
+        }
+      else
+        {
+          result = XFSM_CHOOSE_LOGOUT;
+        }
+
+      gtk_widget_destroy (chooser);
+
+      if (result == XFSM_CHOOSE_NEW)
+        {
+          GtkWidget *button;
+          dialog = gtk_dialog_new_with_buttons (NULL,
+                                                NULL,
+                                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                _("_Cancel"),
+                                                GTK_RESPONSE_CANCEL,
+                                                _("_OK"),
+                                                GTK_RESPONSE_OK,
+                                                NULL);
+          gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK);
+          gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+          gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+          gtk_container_set_border_width (GTK_CONTAINER (dialog), 12);
+          /* Use Adwaita's keycap style to get a meaningful look out of the box */
+          gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (dialog)), "keycap");
+          gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (dialog)), "xfsm-session-chooser");
+          g_snprintf (title, 256, "<big><b>%s</b></big>",
+                      _("Name for the new session"));
+          label = gtk_label_new (title);
+          gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+          gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                              label, TRUE, TRUE, 6);
+          gtk_widget_show (label);
+
+          button = gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog),
+                                                       GTK_RESPONSE_OK);
+          gtk_style_context_add_class (gtk_widget_get_style_context (button), "suggested-action");
+
+          entry = gtk_entry_new ();
+          gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+          gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                              entry, TRUE, TRUE, 6);
+          gtk_widget_show (entry);
+
+again1:
+          result = gtk_dialog_run (GTK_DIALOG (dialog));
+
+          if (result != GTK_RESPONSE_OK)
+            {
+              gtk_widget_destroy (dialog);
+              goto again;
+            }
+
+          if (name_return != NULL)
+            {
+              *name_return = gtk_editable_get_chars (GTK_EDITABLE (entry),
+                                                     0, -1);
+              if (strlen (*name_return) == 0)
+                {
+                  g_free (*name_return);
+                  goto again1;
+                }
+            }
+
+          gtk_widget_destroy (dialog);
+          result = XFSM_CHOOSE_NEW;
+        }
+
+  return result;
+}
+
+
 static void
 xfsm_manager_load_settings (XfsmManager   *manager,
                             XfconfChannel *channel)
@@ -699,7 +755,7 @@ xfsm_manager_load_settings (XfsmManager   *manager,
 
   manager->session_chooser = xfconf_channel_get_bool (channel, "/chooser/AlwaysDisplay", FALSE);
 
-  session_loaded = xfsm_manager_load_session (manager);
+  session_loaded = xfsm_manager_load_session (manager, channel);
 
   if (session_loaded)
     {
@@ -712,12 +768,6 @@ xfsm_manager_load_settings (XfsmManager   *manager,
 
       if (!xfsm_manager_load_failsafe (manager, channel, &errorstr))
         {
-          if (G_LIKELY (splash_screen != NULL))
-            {
-              xfsm_splash_screen_free (splash_screen);
-              splash_screen = NULL;
-            }
-
           /* FIXME: migrate this into the splash screen somehow so the
            * window doesn't look ugly (right now no WM is running, so it
            * won't have window decorations). */
@@ -770,27 +820,11 @@ xfsm_manager_load (XfsmManager   *manager,
 gboolean
 xfsm_manager_restart (XfsmManager *manager)
 {
-  GdkPixbuf *preview;
-  unsigned   steps;
-
   g_assert (manager->session_name != NULL);
 
   /* setup legacy application handling */
   xfsm_legacy_init ();
 
-  /* tell splash screen that the session is starting now */
-  preview = xfsm_load_session_preview (manager->session_name);
-  if (preview == NULL)
-    {
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      /* TODO: Turn this into a normal pixbuf? */
-      preview = gdk_pixbuf_new_from_inline (-1, xfsm_chooser_icon_data, FALSE, NULL);
-      G_GNUC_END_IGNORE_DEPRECATIONS
-    }
-  steps = g_queue_get_length (manager->failsafe_mode ? manager->failsafe_clients : manager->pending_properties);
-  xfsm_splash_screen_start (splash_screen, manager->session_name, preview, steps);
-  g_object_unref (preview);
-
   g_idle_add ((GSourceFunc) xfsm_manager_startup, manager);
 
   return TRUE;
@@ -945,6 +979,9 @@ xfsm_manager_register_client (XfsmManager *manager,
     }
   else
     {
+      xfsm_verbose ("No previous_id found.\n");
+      if (manager->failsafe_mode)
+        xfsm_verbose ("Plus, we're obviously running in failsafe mode.\n");
       if (sms_conn != NULL)
         {
           /* new sms client */
@@ -960,6 +997,9 @@ xfsm_manager_register_client (XfsmManager *manager,
   /* this part is for dbus clients */
   if (dbus_client_id != NULL)
     {
+      xfsm_verbose ("dbus_client_id found: %s.\n", dbus_client_id);
+      if (manager->failsafe_mode)
+        xfsm_verbose ("Plus, we're obviously running in failsafe mode.\n");
       properties = xfsm_manager_get_pending_properties (manager, dbus_client_id);
 
       if (properties != NULL)
@@ -979,6 +1019,8 @@ xfsm_manager_register_client (XfsmManager *manager,
           g_free (hostname);
         }
     }
+  else
+    xfsm_verbose ("No dbus_client_id found.\n");
 
 
   g_queue_push_tail (manager->running_clients, client);
@@ -1000,7 +1042,8 @@ xfsm_manager_register_client (XfsmManager *manager,
         }
     }
 
-  if (previous_id != NULL && manager->state == XFSM_MANAGER_STARTUP)
+  if ((previous_id != NULL || manager->failsafe_mode)
+       && manager->state == XFSM_MANAGER_STARTUP)
     {
       /* Only continue the startup if the previous_id matched one of
        * the starting_properties. If there was no match above,
@@ -1008,6 +1051,11 @@ xfsm_manager_register_client (XfsmManager *manager,
        * in failsafe mode because in that case the failsafe session is
        * started all at once.
        */
+      if (manager->failsafe_mode)
+        {
+          if (--manager->failsafe_clients_pending == 0)
+            g_queue_clear (manager->starting_properties);
+        }
       if (g_queue_peek_head (manager->starting_properties) == NULL)
         xfsm_startup_session_continue (manager);
     }
@@ -1181,7 +1229,7 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
   GList    *lp;
   GError   *error = NULL;
 
-  xfsm_verbose ("entering");
+  xfsm_verbose ("entering\n");
 
   if (shutdown)
     {
@@ -1205,6 +1253,8 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
       if (shutdown_type != XFSM_SHUTDOWN_ASK)
         manager->shutdown_type = shutdown_type;
 
+      xfsm_launch_desktop_files_on_shutdown (FALSE, manager->shutdown_type);
+
       /* we only save the session and quit if we're actually shutting down;
        * suspend, hibernate, hybrid sleep and switch user will (if successful) return us to
        * exactly the same state, so there's no need to save session */
@@ -1297,7 +1347,7 @@ xfsm_manager_save_yourself (XfsmManager *manager,
                             gboolean     fast,
                             gboolean     global)
 {
-  xfsm_verbose ("entering");
+  xfsm_verbose ("entering\n");
 
   if (G_UNLIKELY (xfsm_client_get_state (client) != XFSM_CLIENT_IDLE))
     {
@@ -1341,7 +1391,7 @@ void
 xfsm_manager_save_yourself_phase2 (XfsmManager *manager,
                                    XfsmClient *client)
 {
-  xfsm_verbose ("entering");
+  xfsm_verbose ("entering\n");
 
   if (manager->state != XFSM_MANAGER_CHECKPOINT && manager->state != XFSM_MANAGER_SHUTDOWN)
     {
@@ -1369,7 +1419,7 @@ xfsm_manager_save_yourself_done (XfsmManager *manager,
                                  XfsmClient  *client,
                                  gboolean     success)
 {
-  xfsm_verbose ("entering");
+  xfsm_verbose ("entering\n");
 
   /* In xfsm_manager_interact_done we send SmsShutdownCancelled to clients in
      XFSM_CLIENT_WAITFORINTERACT state. They respond with SmcSaveYourselfDone
@@ -1758,7 +1808,7 @@ xfsm_manager_start_client_save_timeout (XfsmManager *manager,
    * timer, this call will clear it. */
   g_object_set_data_full (G_OBJECT (client), "--save-timeout-id",
                           GUINT_TO_POINTER (sdata->timeout_id),
-                          (GDestroyNotify) g_source_remove);
+                          (GDestroyNotify) G_CALLBACK (g_source_remove));
 }
 
 
@@ -1903,8 +1953,6 @@ xfsm_manager_get_queue (XfsmManager         *manager,
         return manager->restart_properties;
       case XFSM_MANAGER_QUEUE_RUNNING_CLIENTS:
         return manager->running_clients;
-      case XFSM_MANAGER_QUEUE_FAILSAFE_CLIENTS:
-        return manager->failsafe_clients;
       default:
         g_warning ("Requested invalid queue type %d", (gint)q_type);
         return NULL;
@@ -1919,6 +1967,13 @@ xfsm_manager_get_use_failsafe_mode (XfsmManager *manager)
 }
 
 
+void
+xfsm_manager_increase_failsafe_pending_clients (XfsmManager *manager)
+{
+  manager->failsafe_clients_pending++;
+}
+
+
 gboolean
 xfsm_manager_get_compat_startup (XfsmManager          *manager,
                                  XfsmManagerCompatType type)
